@@ -22,6 +22,7 @@
 import signal
 import warnings
 import sys
+import socket
 
 from ..module import get_introspection_module
 from .._gi import (variant_type_from_string, source_new,
@@ -39,15 +40,14 @@ __all__.append('option')
 
 
 # Types and functions still needed from static bindings
-from gi._gi import _glib
-from gi._gi import _gobject
+from gi import _gi
 from gi._error import GError
 
 Error = GError
-OptionContext = _glib.OptionContext
-OptionGroup = _glib.OptionGroup
-Pid = _glib.Pid
-spawn_async = _glib.spawn_async
+OptionContext = _gi.OptionContext
+OptionGroup = _gi.OptionGroup
+Pid = _gi.Pid
+spawn_async = _gi.spawn_async
 
 
 def threads_init():
@@ -250,7 +250,16 @@ class Variant(GLib.Variant):
         return GLib.Variant.new_tuple(elements)
 
     def __del__(self):
-        self.unref()
+        try:
+            self.unref()
+        except ImportError:
+            # Calling unref will cause gi and gi.repository.GLib to be
+            # imported. However, if the program is exiting, then these
+            # modules have likely been removed from sys.modules and will
+            # raise an exception. Assume that's the case for ImportError
+            # and ignore the exception since everything will be cleaned
+            # up, anyways.
+            pass
 
     def __str__(self):
         return self.print_(True)
@@ -467,6 +476,7 @@ def get_string(self):
     value, length = GLib.Variant.get_string(self)
     return value
 
+
 setattr(Variant, 'get_string', get_string)
 
 __all__.append('Variant')
@@ -477,6 +487,8 @@ def markup_escape_text(text, length=-1):
         return GLib.markup_escape_text(text.decode('UTF-8'), length)
     else:
         return GLib.markup_escape_text(text, length)
+
+
 __all__.append('markup_escape_text')
 
 
@@ -539,7 +551,7 @@ for name in ['G_MINFLOAT', 'G_MAXFLOAT', 'G_MINDOUBLE', 'G_MAXDOUBLE',
              'G_MAXUINT', 'G_MINLONG', 'G_MAXLONG', 'G_MAXULONG', 'G_MAXSIZE',
              'G_MINSSIZE', 'G_MAXSSIZE', 'G_MINOFFSET', 'G_MAXOFFSET']:
     attr = name.split("_", 1)[-1]
-    globals()[attr] = getattr(_gobject, name)
+    globals()[attr] = getattr(_gi, name)
     __all__.append(attr)
 
 
@@ -575,6 +587,7 @@ class MainLoop(GLib.MainLoop):
             # caught by _main_loop_sigint_handler()
             raise KeyboardInterrupt
 
+
 MainLoop = override(MainLoop)
 __all__.append('MainLoop')
 
@@ -584,6 +597,7 @@ class MainContext(GLib.MainContext):
     def iteration(self, may_block=True):
         return super(MainContext, self).iteration(may_block)
 
+
 MainContext = override(MainContext)
 __all__.append('MainContext')
 
@@ -633,6 +647,7 @@ class Source(GLib.Source):
 
     can_recurse = property(__get_can_recurse, __set_can_recurse)
 
+
 Source = override(Source)
 __all__.append('Source')
 
@@ -648,6 +663,7 @@ class Idle(Source):
         if priority != GLib.PRIORITY_DEFAULT:
             self.set_priority(priority)
 
+
 __all__.append('Idle')
 
 
@@ -661,6 +677,7 @@ class Timeout(Source):
         if priority != GLib.PRIORITY_DEFAULT:
             self.set_priority(priority)
 
+
 __all__.append('Timeout')
 
 
@@ -669,6 +686,7 @@ def idle_add(function, *user_data, **kwargs):
     priority = kwargs.get('priority', GLib.PRIORITY_DEFAULT_IDLE)
     return GLib.idle_add(priority, function, *user_data)
 
+
 __all__.append('idle_add')
 
 
@@ -676,6 +694,7 @@ def timeout_add(interval, function, *user_data, **kwargs):
     priority = kwargs.get('priority', GLib.PRIORITY_DEFAULT)
     return GLib.timeout_add(priority, interval, function, *user_data)
 
+
 __all__.append('timeout_add')
 
 
@@ -683,6 +702,7 @@ def timeout_add_seconds(interval, function, *user_data, **kwargs):
     priority = kwargs.get('priority', GLib.PRIORITY_DEFAULT)
     return GLib.timeout_add_seconds(priority, interval, function, *user_data)
 
+
 __all__.append('timeout_add_seconds')
 
 
@@ -726,6 +746,9 @@ def _io_add_watch_get_args(channel, priority_, condition, *cb_and_user_data, **k
     if isinstance(channel, int):
         func_fdtransform = lambda _, cond, *data: callback(channel, cond, *data)
         real_channel = GLib.IOChannel.unix_new(channel)
+    elif isinstance(channel, socket.socket) and sys.platform == 'win32':
+        func_fdtransform = lambda _, cond, *data: callback(channel, cond, *data)
+        real_channel = GLib.IOChannel.win32_new_socket(channel.fileno())
     elif hasattr(channel, 'fileno'):
         # backwards compatibility: Allow calling with Python file
         func_fdtransform = lambda _, cond, *data: callback(channel, cond, *data)
@@ -737,6 +760,7 @@ def _io_add_watch_get_args(channel, priority_, condition, *cb_and_user_data, **k
 
     return real_channel, priority_, condition, func_fdtransform, user_data
 
+
 __all__.append('_io_add_watch_get_args')
 
 
@@ -745,6 +769,7 @@ def io_add_watch(*args, **kwargs):
     channel, priority, condition, func, user_data = _io_add_watch_get_args(*args, **kwargs)
     return GLib.io_add_watch(channel, priority, condition, func, *user_data)
 
+
 __all__.append('io_add_watch')
 
 
@@ -826,6 +851,7 @@ class IOChannel(GLib.IOChannel):
     # Python 2.x compatibility
     next = __next__
 
+
 IOChannel = override(IOChannel)
 __all__.append('IOChannel')
 
@@ -840,6 +866,7 @@ class PollFD(GLib.PollFD):
         self.fd = fd
         self.events = events
 
+
 PollFD = override(PollFD)
 __all__.append('PollFD')
 
@@ -887,6 +914,7 @@ def _child_watch_add_get_args(priority_or_pid, pid_or_callback, *args, **kwargs)
 
     return priority, pid, callback, user_data
 
+
 # we need this to be accessible for unit testing
 __all__.append('_child_watch_add_get_args')
 
@@ -896,12 +924,14 @@ def child_watch_add(*args, **kwargs):
     priority, pid, function, data = _child_watch_add_get_args(*args, **kwargs)
     return GLib.child_watch_add(priority, pid, function, *data)
 
+
 __all__.append('child_watch_add')
 
 
 def get_current_time():
     return GLib.get_real_time() * 0.000001
 
+
 get_current_time = deprecated(get_current_time, 'GLib.get_real_time()')
 
 __all__.append('get_current_time')
@@ -912,6 +942,7 @@ __all__.append('get_current_time')
 def filename_from_utf8(utf8string, len=-1):
     return GLib.filename_from_utf8(utf8string, len)[0]
 
+
 __all__.append('filename_from_utf8')
 
 
